# react-connect-context-hooks

## Install

```bash
npm install --save react-connect-context-hooks
```

## What?

`react-connect-context-hooks` allows you to implement your own state-management solution using React.hooks, while leverages good practices and provide DRY code.

 This library takes some ideas and patterns from `redux`, such as:
 - Defining different `reducers` per feature/module.
 - Defining `actionCreators` to dispatch actions (actions follows `redux-thunk` approach).
 - Providing a `connect` like functionality to allow your components be agnostic from the global state dependencies and receive only what they need.

 Also, provides some good practices around implementing state-management with hooks:
 - Allows hiding your `Context`; it provides custom `Provider` HOC you can use instead for wrapping your components.
 - Provides custom `useContext` hook that allows selecting what you need from the store (similar to the `connect` HOC).
 - Provides different `selection` or mapping mechanisms for easily getting what you need from the `store`.

## Why?

Since React hooks are available, a lot of discussion among the developer-community arises regarding if some complex/external tooling (such as `redux`) is needed to implement state management in your apps.

While is it possible to implement redux-like functionality just with `useReducer` + `useContext`, there are good recommendations and practices that is good to consider, specially if your application grows.

This library allows implementing good practices for your custom state-management solution using hooks, while allowing to avoid repeating boilerplate for every `Provider/Consumer` you need to create.

## How?

`createContextProvider` receives a reducer function, a initial-state object, and a key-value object with actions; it will return a `Provider` component that will be used to wrap your application wherever you need to provide access to context. It will return the `Context` object to you can use to create custom `connect` HOC or `useContext`.

## Usage:

Take a look at the following example:

```js
/// CounterProvider.tsx

import counterReducer, { initialState } from './counterReducer';
import { incrementAction, decrementAction } from './counterActions';

import createContextProvider, { connectContextFactory, useConnectedContextFactory } from 'react-connect-context-hooks';

const [CounterProvider, CounterContext] = createContextProvider(counterReducer, initialState, actions);

const withCounter = connectContextFactory(CounterContext);
const useCounter = useConnectedContextFactory(CounterContext);

export default CounterProvider;
export {
    withCounter,
    useCounter,
};
```

Here we created a `Provider` component and two helper functions:
- `withCounter` is a HOC for connecting your app components as consumers of this context (this is recommended for keeping components isolated from the store).
- `useCounter` can be used as a hook function to have access to the connected context (use this in cases where you can't use the HOC).

The `Provider` can be used to wrap your applications code:

```js
/// App.tsx

import React from 'react';

import CounterProvider from './counter/provider';
import Counter from './counter/Counter';

const App: React.FC = () => {
  return (
    <CounterProvider>
      <Counter />
    </CounterProvider>
  );
}

export default App;
```

Finally, use the `connected` HOC in your components code as follows:

```js
/// CounterComponent.tsx

import React from 'react';

import { withCounter } from './provider/CounterProvider';

const Counter: React.FC<any> = ({ count, increment, decrement }) => (
  <div>
    <h1>Counter Component</h1>
    <p>
      <b>Count: </b>
      <span>{count}</span>
    </p>
    <button onClick={() => decrement(amount)}>Decrement</button>
    <button onClick={() => increment(amount)}>Increment</button>
  </div>
)

// Export isolated component is recommended for unit-testing
export {
  Counter,
}

// Export the connected component as default
export default withCounter(Counter as React.ComponentType<any>, {
  stateMappers: ['count'],
  actionMappers: ['increment', 'decrement'],
});

```

Here we are defining a basic `Counter` component that needs to receive 3 properties: count, increment and decrement.

We are selecting this values from the state and the actions that we have available in our Context.

The `withCounter` function is the one created before, and allows creating a HOC that is bound to the right `CounterContext`. 

This HOC function receives an object with options; in the example we are using these options to select data and actions from the store.

If you need to access the store from a hook, then you would use:

```js
/// useUpdateCounterHook.ts

export default () => {
  const { count, increment } = useCounter({
    stateMappers: ['count'],
    actionMappers: ['increment'],
  });

  if (window.confirm(`Current count value is: ${count}. Would you like to increment it?`)) {
    increment();
  }
}

```

As you can see, `useCounter` receives the same `options` object that the HOC, so you can pick what to select from the store.


**And this is it..** Take a look at the `example` folder to check how to define the `actions` and `reducer`. Also, check below the API documentations to learn more about selectors.


## 


## License

MIT © [adrian-marcelo-gallardo](https://github.com/adrian-marcelo-gallardo)
